@@ -17,11 +17,10 @@ print "ok 1\n";
 # Insert your test code below (better if it prints "ok 13"
 # (correspondingly "not ok 13") depending on the success of chunk 13
 # of the test code):
-my $openssl = new OpenCA::OpenSSL;
+my $openssl = new OpenCA::OpenSSL( SHELL => "/usr/bin/openssl");
 
 my $k = {
-	SHELL => "/usr/local/bin/openssl",
-	CONFIG => "/usr/local/OpenCA/stuff/openssl.cnf"
+	CONFIG => "/usr/ssl/openssl.cnf"
 	};
 
 $openssl->setParams( $k );
